#include "testcode.h"

/**
 * clang 公式ドキュメント UnknownBehaviorSanitizer 
 * https://releases.llvm.org/17.0.1/tools/clang/docs/ReleaseNotes.html
 * 
 * 関数名は呼び出し元のhandler関数名としています。
 * fsanitize_call_XXXX_XXXXX_XXXX
 *  
 */

/**
 * fsanitize_call_ubsan_handle_type_mismatch_v1
 * 
 * 関数説明：NULLポインタを参照させる
 * call handler：__ubsan_handle_type_mismatch_v1
 * 検出可option：-fsanitize=undefined
*/
int fsanitize_call_ubsan_handle_type_mismatch_v1(int n)
{
    int *null_ptr = NULL;
    // NULLポインターを参照して値を読み取る(無効なアクセス)
    int value = *null_ptr;
        
    return value;
}

/**
 * fsanitize_call_ubsan_handle_add_overflow
 * 
 * 関数説明：整数加算オーバーフローを発生させる
 * call handler： __ubsan_handle_add_overflow
 * 検出可option：-fsanitize=unsigned-integer-overflow
 * 検出不可option：-fsanitize=undefined
*/
unsigned long fsanitize_call_ubsan_handle_add_overflow(int n)
{

  unsigned long uli, ulj, ulk;

  ulj = ULONG_MAX;
  ulk = 1;
  uli = ulj + ulk;

  return uli;
}

/**
 * fsanitize_call_ubsan_handle_sub_overflow
 * 
 * 関数説明：整数減算オーバーフローを発生させる
 * call handler：__ubsan_handle_sub_overflow
 * 検出可option：-fsanitize=unsigned-integer-overflow
 * 検出不可option：-fsanitize=undefined
*/
unsigned long fsanitize_call_ubsan_handle_sub_overflow(int n)
{

  unsigned long uli, ulj, ulk;

  ulj = 0;
  ulk = 1;
  uli = ulj - ulk;

  return uli;
}
/**
 * fsanitize_call_ubsan_handle_mul_overflow
 * 
 * 関数説明：整数乗算オーバーフローを発生させる
 * call handler：__ubsan_handle_mul_overflow
 * 検出可option：-fsanitize=unsigned-integer-overflow
 * 検出不可option：-fsanitize=undefined
*/
unsigned long fsanitize_call_ubsan_handle_mul_overflow(int n)
{

  unsigned long uli, ulj, ulk;

  ulj = ULONG_MAX;
  ulk = 2;
  uli = ulj * ulk;

  return uli;
}

/**
 * fsanitize_call_ubsan_handle_shift_out_of_bounds
 * 
 * 関数説明： ビットシフト演算無効または未定義のシフト操作
 * call handler： __ubsan_handle_shift_out_of_bounds
 * 検出可option   ：-fsanitize=undefined,-fsanitize=shift
*/
signed int fsanitize_call_ubsan_handle_shift_out_of_bounds(int n)
{
  signed int ij, ik; //32bit singed int型を宣言
  ik = 1;

  ij = ik << 31; // データ型のビット以上のシフト量(符号ビットへのシフト)@runtime error
  //1 <<  31   10000000000000000000000000000000:dec (-2147483648)

  return ij;
}
/** 
 * fsanitize_call_ubsan_handle_out_of_bounds
 * 
 * 関数説明： 配列やポインタの範囲外アクセスを検出させる
 * call handler： __ubsan_handle_out_of_bounds
 * 検出可option   ：-fsanitize=undefined
*/
void fsanitize_call_ubsan_handle_out_of_bounds(int n)
{
  
  int     buf[10]; 
  int     i = 0;

  while( i < 10 ) { 
    i++;         //i インクリメント
    buf[i] = i;  //配列bufにiを代入処理
  }
}

int main(void) 
{
	return 0;
}
