@REM Copyright (c) 2023, Arm Limited and affiliates.
@REM SPDX-License-Identifier: Apache-2.0
@REM
@REM Licensed under the Apache License, Version 2.0 (the "License");
@REM you may not use this file except in compliance with the License.
@REM You may obtain a copy of the License at
@REM
@REM     http://www.apache.org/licenses/LICENSE-2.0
@REM
@REM Unless required by applicable law or agreed to in writing, software
@REM distributed under the License is distributed on an "AS IS" BASIS,
@REM WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@REM See the License for the specific language governing permissions and
@REM limitations under the License.

@if [%1]==[] goto :target_empty
@set target=%1
@goto :make
:target_empty
@set target=build

:make
@if [%target%]==[build] goto :build
@if [%target%]==[build-minimal] goto :build-minimal
@if [%target%]==[build-trap] goto :build-trap
@if [%target%]==[run] goto :run
@if [%target%]==[clean] goto :clean
@echo Error: unknown target "%target%"
@exit /B 1

:build
@if [%BIN_PATH%]==[] goto :bin_path_empty
@call :build_fn
@exit /B

:build-minimal
@if [%BIN_PATH%]==[] goto :bin_path_empty
@call :build_minimal
@exit /B

:build-trap
@if [%BIN_PATH%]==[] goto :bin_path_empty
@call :build_trap_fn
@exit /B

:run
@if exist a.hex goto :do_run
@if [%BIN_PATH%]==[] goto :bin_path_empty
@call :build_fn
:do_run
qemu-system-arm.exe -M microbit -semihosting -nographic -device loader,file=a.hex
@exit /B

:clean
if exist *.elf del /q *.elf
if exist *.elf.xlo del /q *.elf.xlo
@exit /B

:bin_path_empty
@echo Error: BIN_PATH environment variable is not set
@exit /B 1

:build_fn
%BIN_PATH%\clang.exe --target=arm-none-eabi -mcpu=cortex-r4 -mfloat-abi=soft -O0 -lsemihost -gdwarf-4 -fsanitize=undefined,unsigned-integer-overflow -g -T ../../ldscripts/microbit.ld -o a.elf *.c *.s
@exit /B

:build_minimal
%BIN_PATH%\clang.exe --target=arm-none-eabi -mcpu=cortex-r4 -mfloat-abi=soft -O0 -lsemihost -gdwarf-4 -fsanitize=undefined -fsanitize-minimal-runtime -g -T ../../ldscripts/microbit.ld -o a.elf *.c
@exit /B

:build_trap_fn
%BIN_PATH%\clang.exe --target=arm-none-eabi -mcpu=cortex-r4 -mfloat-abi=soft -O0 -lsemihost -gdwarf-4 -fsanitize=undefined -fsanitize-trap=all -g -T ../../ldscripts/microbit.ld -o a.elf *.c
@exit /B
